/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#ifndef FILEOBSERVER_H_
#define FILEOBSERVER_H_

#include "ClientPush.h"

using namespace Funambol;

/**
 * Observer on file system, listens to files changes on a defined directory
 * and provides callbacks to ClientPush class when a change is detected.
 */
class CFileObserver : public CActive
{
    
private:
    
    /// Pointer to parent class, for callbacks.
    CClientPush* iClientPush;
    
    /**
     * The ID of this FileObserver object.
     * Each object has an ID, set by constructor, to be identified by
     * the parent class (there may be many FileObserver object).
     * The ID is used in the callbacks as a parameter.
     */
    TInt ID;
    
    /// True if is currently listening to modifications.
    bool iListening;
    
    
    /// FileSystem session.
    RFs iFsSession;
    
    /// The directory to listens to files changes.
    StringBuffer iPath;


    /**
     * Handles active object's request completion events.
     * It's called by the active scheduler when a request completion event occurs.
     */
    void RunL(void);
    
    /**
     * Implements cancellation of  outstanding request.
     * This function is called as part of the active object's Cancel().
     */
    void DoCancel(void);
    
    /// Called by the active scheduler if the active object's RunL() function leaves.
    TInt RunError(TInt aError);
    

public:
    
    /**
     * Constructor. MUST NOT leave.
     * @param parent  the parent CClientPush pointer, for callbacks
     * @param aID     the ID for this object
     */
    CFileObserver(CClientPush* parent, const TInt aID);

    /// Destructor
    ~CFileObserver();
    
    
    /// Starts the observer for files modifications.
    void start();
    
    /// Stops the observer for files modifications.
    void stop();

    
    /// Returns true if observer is active (we're listening for notifications).
    bool isRunning() { return iListening; }
    
    /**
     * Sets the path where files are stored, to watch for file changes.
     * If it was already running on a different path, the notification 
     * listener will be stopped and restarted on the new location.
     * @param aPath  the absolute path (like "C:\Data\Images\")
     */
    void setPath(const StringBuffer& aPath);

};

#endif /*FILEOBSERVER_H_*/
