/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008-2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef MEDIAOPTIONSDIALOG_H_
#define MEDIAOPTIONSDIALOG_H_

#include <akndialog.h> 
#include <aknlists.h>
#include <aknsettingitemlist.h> 

#include "base/util/ArrayList.h"
#include "MediaOptionsSettingList.h"

// FORWARD DECLARATIONS
class CFunambolAppUi;

using namespace Funambol;


/**
 * The media options dialog (Pictures or Videos).
 */
class CMediaOptionsDialog : public CAknDialog
{

private:
    
    /// Pointer to the object that created this instance, the appUI.
    /// Used to callback methods to the appUI, which is the controller.
    CFunambolAppUi*             iAppUI;
    
    CMediaOptionsSettingList*  iSettingList;
    
    /// Defines the source ID corresponding to this dialog: Pictures or Videos.
    TInt                        iSourceID;
    
    /// Useful to store also the name, once for all.
    StringBuffer                iSourceName;
    
    /// ArrayList of available sync modes (StringBuffer elements) for this source.
    ArrayList                   iSyncModes;
    
    TInt                        iSyncMode;
    TInt                        iMemoryInUse;
    TBool                       iMassStorageAvailable;
    TBool                       iIncludeOlderItems;

    // Default constructor/destructor
    CMediaOptionsDialog();
    ~CMediaOptionsDialog();
    
    /// Calls the CAknDialog::ConstructL, with the menu resource.
    void ConstructL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID);
    
    
    /// Overrided to create the setting list at runtime. 
    void PreLayoutDynInitL();
    
    // Overrided to handle dialog's menu commands.
    // Not used: we don't use a menu!
    //void ProcessCommandL(TInt aCommand);
    
    /// Invoked before leaving the dialog. Decides if we need to save or cancel.
    TBool OkToExitL(TInt aButtonId);
    
    /// Overrided to capture the key events: up/down arrow, selection key.
    TKeyResponse OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType);
    
    /**
     * Execute operations to edit the current item.
     * Pictures sync direction is fixed, so it won't open up.
     */
    //void EditCurrentItem();
    
    /**
     * To show/hide the right UI items.
     * - filtering: show the right filtering item for this source
     * - storage:   show the binary/enumerated item based on the drives availabilities
     */
    void RefreshDynamicItems();
    
    /**
     * Loads data from client configuration into the UI dialog.
     */
    void  load();
    
    /**
     * Saves data from UI dialog into the client configuration.
     * If data is unchanged, exits with EFalse.
     * @return  ETrue if settings saved correctly, EFalse if not.
     */
    TBool save();
    
    /**
     * Called on startup, to fill the syncModes array with supported sync modes
     * for this source. The "slow" mode can't be selected by the user and
     * the "none" mode is added at the beginning of the array (the "Do nothing" action).
     */
    void prepareSyncModesList(const StringBuffer& modes);
    
    /**
     * Reads from the resources and returns an array of SyncMode labels corresponding
     * to the iSyncModes list of available sync modes.
     * Like: "none" -> "Do Nothing", "two-way" -> "Sync Two Way", ...
     * @return  new allocated ArrayList* of StringBuffer objects
     */
    ArrayList* getSyncModeLabels();
    
public:
    
    /**
     * Creates a new CPhotoOptionsDialog object.
     * @param aMenuTitleResourceId  the resource ID of the menubar for this dialog
     * @param aAppUI                pointer to the FunambolAppUI object that called this method
     */
    static CMediaOptionsDialog* NewL(TInt aMenuTitleResourceId, CFunambolAppUi* aAppUI, const TInt aSourceID);  
};

#endif /*MEDIAOPTIONSDIALOG_H_*/
