/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef MEDIA_OPTIONS_SETTING_LIST
#define MEDIA_OPTIONS_SETTING_LIST


#include "CommonOptionsSettingList.h"

using namespace Funambol;


/**
 * This class is used to rapresent the setting list of controls
 * in all options dialogs (emailOptions / PIMOptions / SyncMethod).
 */
class CMediaOptionsSettingList : public CCommonOptionsSettingList
{

private:

    TInt    iSourceID;
    TInt    iSyncMode;

    TInt    iMemoryInUseBinary;
    TInt    iMemoryInUseMultiple;
    TBool   iIncludeOlderPictures;
    TBool   iIncludeOlderVideos;


public:

    CMediaOptionsSettingList(const TInt sourceID);

    /**
     * Overrides the method of CCommonOptionsSettingList.
     */
    CAknSettingItem* CreateSettingItemL(TInt aIdentifier);

    /**
     * Creates dynamically the SyncMode selection item as a list
     * of choices, given the passed list of labels.
     */
    void ConstructSyncModeList(ArrayList* syncModeLabels);


    //
    // ------ get/set methods ------
    //
    TInt         getSyncMode()             { return iSyncMode;                    }
    TInt         getMemoryInUseBinary()    { return iMemoryInUseBinary;           }
    TInt         getMemoryInUseMultiple()  { return iMemoryInUseMultiple;         }
    TBool        getIncludeOlderPictures() { return iIncludeOlderPictures;        }
    TBool        getIncludeOlderVideos()   { return iIncludeOlderVideos;          }

    void setSyncMode            (const TInt aSyncMode)    { iSyncMode            = aSyncMode;    }
    void setMemoryInUseBinary   (const TInt aMemoryInUse) { iMemoryInUseBinary   = aMemoryInUse; }
    void setMemoryInUseMultiple (const TInt aMemoryInUse) { iMemoryInUseMultiple = aMemoryInUse; }
    void setIncludeOlderPictures(const TBool aIncludeOlderPictures) { iIncludeOlderPictures = aIncludeOlderPictures; }
    void setIncludeOlderVideos  (const TBool aIncludeOlderVideos)   { iIncludeOlderVideos   = aIncludeOlderVideos;   }

};

#endif  // MEDIA_OPTIONS_SETTING_LIST

