/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// INCLUDE FILES
#include <coemain.h>
#include "FunambolAppView.h"

#include <eikenv.h>
#include <eikon.hrh>
#include <e32cmn.h>
#include <akniconarray.h>
#include <aknsdrawutils.h>
#include <aknsbasicbackgroundcontrolcontext.h>

#include "uid3.h"
#include "base/util/symbianUtils.h"
#include "base/SymbianLog.h"
#include "commonDefs.h"
#include "customization.h"
#include "utils.h"
#include "ClientSettings.h"



CFunambolAppView* CFunambolAppView::NewL(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    CFunambolAppView* self = CFunambolAppView::NewLC( aRect, aAppUI );
    CleanupStack::Pop( self );
    return self;
}

CFunambolAppView* CFunambolAppView::NewLC(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    CFunambolAppView* self = new ( ELeave ) CFunambolAppView;
    CleanupStack::PushL( self );
    self->ConstructL( aRect, aAppUI );
    return self;
}

void CFunambolAppView::ConstructL(const TRect& aRect, CFunambolAppUi* aAppUI)
{
    iAppUI = aAppUI;

    // Create a window for this application view
    CreateWindowL();

    // Creates the listbox of icon+label.
    TRAPD(err, CreateListBoxL(aRect);)
    if (err) {
        LOG.error("Error creating Listbox: code %d", err);
        User::Leave(err);   // Forward the leave
    }

    // Set the windows size
    SetRect( aRect );

    iAppUI->SetListboxFontSize(iListBox->ItemDrawer());

    iBgContext = CAknsBasicBackgroundControlContext::NewL( KAknsIIDQsnBgAreaMain,aRect,ETrue);

    // Activate the window, which makes it ready to be drawn
    ActivateL();
}


// C++ default constructor can NOT contain any code, that might leave.
CFunambolAppView::CFunambolAppView()
{
    iListboxItems.clear();
}

// Destructor.
CFunambolAppView::~CFunambolAppView()
{
    delete iBgContext;
}


void CFunambolAppView::Draw( const TRect& aRect ) const
{
    // Get the standard graphics context
    CWindowGc& gc = SystemGc();

    // Redraw the background using the default skin
    MAknsSkinInstance* skin = AknsUtils::SkinInstance();
    MAknsControlContext* cc = AknsDrawUtils::ControlContext( this );
    AknsDrawUtils::Background( skin, cc, this, gc, aRect );

    // Gets the control's extent
    TRect drawRect( Rect() );

    //iListBox->SetItemHeightL(25);
    //iListBox->View()->SetItemHeight(25);
    // Clears the screen
    //gc.Clear( drawRect );
}

void CFunambolAppView::SizeChanged()
{
    if(iBgContext) {
        iBgContext->SetRect(Rect());
        if ( &Window() ) {
            iBgContext->SetParentPos( PositionRelativeToScreen() );
        }
    }
    if (iListBox) {
        iListBox->SetExtent(TPoint(0,0), iListBox->MinimumSize() );
    }

    iAppUI->SetListboxFontSize(iListBox->ItemDrawer());

    DrawNow();
}

TTypeUid::Ptr CFunambolAppView::MopSupplyObject(TTypeUid aId)
{
    if (iBgContext ) {
       return MAknsControlContext::SupplyMopObject( aId, iBgContext );
    }
    return CCoeControl::MopSupplyObject(aId);
}


void CFunambolAppView::CreateListBoxL(const TRect&/* aRect*/)
{
    //
    // Create the UI ListBox
    //
    if (USE_MAIN_SCREEN_BIG_ICONS) {
        iListBox = new (ELeave) CAknDoubleLargeStyleListBox();
    } else {
        iListBox = new (ELeave) CAknDoubleGraphicStyleListBox();
    }
    iListBox->SetContainerWindowL(*this);
    
    // Load ListBox data from static resource (only 1 is set)
    TResourceReader rr;
    iCoeEnv->CreateResourceReaderLC(rr, R_SYNC_LIST);
    iListBox->ConstructFromResourceL(rr);
    
    
    // array of ALL sources that have a UI
    allSourcesWithUI.clear();
    StringBuffer allSourceOrder(SOURCES_WITH_UI);
    allSourceOrder.split(allSourcesWithUI, ",");
    
    //
    // Fill the array of ALL svg icons (2 for each source that have a UI)
    //
    int totIcons = allSourcesWithUI.size() * 2;
    CArrayPtr<CGulIcon>* icons = new(ELeave) CAknIconArray(totIcons);
    CleanupStack::PushL(icons);
    
    for (int i=0; i<allSourcesWithUI.size(); i++) {
        StringBuffer* name = (StringBuffer*)allSourcesWithUI[i];
        TInt sourceID = getSourceIDFromName(*name);
        
        appendIcon(icons, getSourceIconId(sourceID));
        appendIcon(icons, getSourceIconDimmedId(sourceID));
    }

    // Set the icons array to the listbox
    iListBox->ItemDrawer()->ColumnData()->SetIconArray(icons);
    CleanupStack::Pop();              // icons
    CleanupStack::PopAndDestroy();    // rr

    // Remove ALL sources from the UI ListBox: they will be added dynamically
    // (see method 'refresh()')
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList = static_cast<CDesCArray*>(textArray);
    itemList->Reset();
    iListBox->HandleItemAdditionL();

    
    // Adds the scrollbar (TODO: EAuto visiblity seems not working...)
    iListBox->CreateScrollBarFrameL(ETrue);
    //iListBox->ScrollBarFrame()->SetScrollBarVisibilityL(CEikScrollBarFrame::EAuto, CEikScrollBarFrame::EAuto);
    
    // The list box model owns the 'iListBox' item array, and will delete 
    // it in its destructor !! (crazy guy designed this)
    iListBox->Model()->SetOwnershipType(ELbmOwnsItemArray);
}


void CFunambolAppView::refresh()
{
    ClientSettings* config = ClientSettings::getInstance();
    const ArrayList& sourcesVisible = config->getSourcesVisible();
    
    //
    // Fill the iListboxItems array with sources visible, in the right order
    //
    LOG.debug("--- refresh UI panes ---");
    for (int i=0; i<sourcesVisible.size(); i++) {
        
        StringBuffer* name = (StringBuffer*)sourcesVisible[i];
        SyncSourceConfig* ssc = config->getSyncSourceConfig(*name);
        if (!ssc) { continue; }
        TInt sourceID = getSourceIDFromName(*name);
        
        ListboxItem* oldItem = (ListboxItem*)iListboxItems.get(i);
        if (oldItem && oldItem->getSourceId() == sourceID) {
            //
            // same item in the same position: update it
            //
            oldItem->setEnabled(ssc->isEnabled());
            LOG.debug("       %d) %s, enabled=%d, iconsIx=%d,%d", i+1, name->c_str(),  
                    oldItem->getEnabled(), oldItem->getIconIndex(), oldItem->getIconDimmedIndex());
            continue;
        }
        
        bool found = false;
        for (int j=i+1; j<iListboxItems.size(); j++) {
            oldItem = (ListboxItem*)iListboxItems.get(j);
            if (oldItem && oldItem->getSourceId() == sourceID) {
                //
                // found same item in a different position: move it (j -> i)
                //
                ListboxItem item(*oldItem);
                iListboxItems.removeElementAt(j);
                item.setEnabled(ssc->isEnabled());
                
                iListboxItems.add(i, item);
                LOG.debug("[mov]  %d) %s, enabled=%d, iconsIx=%d,%d (old pos was %d)", i+1, name->c_str(),  
                          item.getEnabled(), item.getIconIndex(), item.getIconDimmedIndex(), j);
                found = true;
                break;
            }
        }
        if (found) {
            continue;
        }
        
        //
        // not found: append a new item
        //
        StringBuffer label = iAppUI->getSourceLabel(sourceID);
        bool enabled       = ssc->isEnabled();
        int iconIndex      = getIconIndex(sourceID);

        ListboxItem item(sourceID, label, " ", enabled, iconIndex, iconIndex+1);
        iListboxItems.add(item);
        LOG.debug("[new]  %d) %s, enabled=%d, iconsIx=%d,%d", i+1, name->c_str(), 
                  item.getEnabled(), item.getIconIndex(), item.getIconDimmedIndex());
    }
    
    //
    // remove exceeding items from the end of the list
    //
    for (int i=iListboxItems.size(); i>sourcesVisible.size(); i--) {
        LOG.debug("removing exceeding item at pos = %d", i-1);
        iListboxItems.removeElementAt(i-1);
    }
    
    
    // Now the ListBox array is up to date: refresh the screen
    refreshAllPanes();
}

ListboxItem* CFunambolAppView::getListboxItem(const TInt aSourceID)
{
    int index = 0;
    return getListboxItem(aSourceID, &index);
}

ListboxItem* CFunambolAppView::getListboxItem(const TInt aSourceID, int* index)
{
    for (int i=0; i<iListboxItems.size(); i++) {
        ListboxItem* item = (ListboxItem*)iListboxItems[i];
        if (item->getSourceId() == aSourceID) {
            *index = i;
            return item;
        }
    }
    
    // not found
    *index = KErrNotFound;
    return NULL;
}


TInt CFunambolAppView::CountComponentControls() const
{
    return 1;
}

CCoeControl* CFunambolAppView::ComponentControl(TInt aIndex) const
{
    switch ( aIndex )
    {
        case 0:
            return iListBox;
        default:
            return NULL;
    }
}

TKeyResponse CFunambolAppView::OfferKeyEventL(const TKeyEvent &aKeyEvent, TEventCode aType)
{
    if ( aType != EEventKey ){
        return EKeyWasNotConsumed;
    }

//    LOG.debug("CFunambolAppView::OfferKeyEventL - key pressed: %04x", aKeyEvent.iCode);
//    _LIT(msgFormat, "key pressed: code 0x%04x, scanCode = 0x%02x");
//    TBuf<100> msg;
//    msg.Format(msgFormat, aKeyEvent.iCode, aKeyEvent.iScanCode);
//    msgBox(msg);

    if (aKeyEvent.iScanCode == EStdKeyDevice3) {        // ???????? why this code ????????
        // Manage the actions on the AppUI.
        iAppUI->OnFireButton();
        return EKeyWasConsumed;
    }

    // Fix: refresh the main buttons when a key is pressed.
    // In case of dirty cba buttons, we can restore the correct button text.
    iAppUI->refreshMainButtons();

    switch ( aKeyEvent.iCode ){
        // Up & Down arrow key's event transfer to list box
        case EKeyDownArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        case EKeyUpArrow:
            return iListBox->OfferKeyEventL( aKeyEvent, aType );
            break;
        default:
            break;
    }
    return EKeyWasNotConsumed;
}



void CFunambolAppView::RefreshSourceStatus(const TInt aSourceID, const TDesC& aText, const TBool aDimmed)
{
    //LOG.debug("RefreshSourceStatus (source %d, dimmed = %d)", aSourceID, aDimmed);
            
    int index = KErrNotFound;
    ListboxItem* item = getListboxItem(aSourceID, &index);
    if (!item || index<0) {
        return;
    }
    
    // Store the current status and dimmed information.
    StringBuffer text = bufToStringBuffer(aText);
    item->setStatus(text);
    item->setEnabled(!aDimmed);

    refreshPane(item, index);
}

void CFunambolAppView::RefreshSourceBufferedStatus(const TInt aSourceID, const TBool aDimmed)
{
    //LOG.debug("RefreshSourceBufferedStatus (source %d, dimmed = %d)", aSourceID, aDimmed);
    
    int index = KErrNotFound;
    ListboxItem* item = getListboxItem(aSourceID, &index);
    if (!item || index<0) {
        return;
    }
    
    item->setEnabled(!aDimmed);
    refreshPane(item, index);
}


void CFunambolAppView::refreshAllPanes()
{
    //
    // Copy the content of iListboxItems array to UI
    //
    for (int i=0; i<iListboxItems.size(); i++) {
        ListboxItem* item = (ListboxItem*)iListboxItems[i];
        refreshPane(item, i);
    }
    
    // 
    // Remove exceeding UI items
    //
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    TInt numItems = itemList->Count();
    
    if (numItems > iListboxItems.size()) {
        iCriticalListboxText.Wait();
        for (int i=numItems; i>iListboxItems.size(); i--) {
            itemList->Delete(i-1);
        }
        iListBox->HandleItemAdditionL();
        iCriticalListboxText.Signal();
    }
}


void CFunambolAppView::refreshPane(ListboxItem* item, const int itemIndex) 
{
    //LOG.debug("CFunambolAppView::refreshPane (index = %d)", itemIndex);
    if (!item || itemIndex < 0) {
        return;
    }
    
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    TInt numItems = itemList->Count();
    
    // Get the right icon index
    TInt iconIndex = 0;
    if (item->getEnabled()) {
        iconIndex = item->getIconIndex();
    } else {
        iconIndex = item->getIconDimmedIndex();
    }
    
    //
    // Format the text of the listbox item.
    // It is a predefined formatting, see CEikFormattedCellListBoxTypedef documentation.
    //
    StringBuffer textLine;
    textLine.sprintf("%d\t%s\t%s\t", iconIndex, item->getLabel().c_str(), item->getStatus().c_str());
    //LOG.debug("textLine = '%s'", textLine.c_str());

    //
    // Update the UI listbox
    //
    iCriticalListboxText.Wait();
    
    int pos = itemIndex;
    if (pos >= numItems) {
        // append at the end
        pos = itemList->Count();
    }
    else {
        // Replace the item at this position
        //LOG.debug("delete item at pos = %d", pos);
        itemList->Delete(pos);
    }

    // Insert the new item
    TRAPD( err, {
           //LOG.debug("insert item at pos = %d", pos);
           HBufC* buf = stringBufferToNewBuf(textLine);
           itemList->InsertL(pos, *buf);
           iListBox->HandleItemAdditionL();        // To update the listbox
           delete buf;
           } )
    if (err) {
        LOG.error("refreshPane error inserting item: code %d", err);
    }

    iCriticalListboxText.Signal();
}

bool CFunambolAppView::isSourceDisplayed(const TInt aSourceID) 
{ 
    return getListboxItem(aSourceID)? true : false;
}

int CFunambolAppView::countSourcesDisplayed() 
{
    return iListboxItems.size();
}

TInt CFunambolAppView::GetCurrentItemSelected()
{
    return iListBox->CurrentItemIndex();
}

TInt CFunambolAppView::GetCurrentSourceSelected()
{
    TInt pos = GetCurrentItemSelected();
    if (pos >= 0 && pos < iListboxItems.size()) {
        ListboxItem* item = (ListboxItem*)iListboxItems[pos];
        return item->getSourceId();
    }
    return KErrNotFound;
}

void CFunambolAppView::scrollToSourcePane(const TInt aSourceID) {
    
    int index = KErrNotFound;
    getListboxItem(aSourceID, &index);
    
    if (index != KErrNotFound) {
        iListBox->ScrollToMakeItemVisible(index);
    }
}


int CFunambolAppView::getIconIndex(const TInt aSourceID) {
    
    // array of ALL sources that have a UI
    for (int i=0; i<allSourcesWithUI.size(); i++) {
        StringBuffer* name = (StringBuffer*)allSourcesWithUI[i];
        TInt sourceID = getSourceIDFromName(*name);
        if (aSourceID == sourceID) {
            return i*2;                 // 2 icons for every source with UI (normal & dimmed).
        }
    }
    // not found
    return KErrNotFound;
}


/*
int CFunambolAppView::appendSourcePane(const TInt aSourceID) {
    
    LOG.debug("adding source pane %s", getSourceNameFromID(aSourceID).c_str());
    return appendSourcePane(getListboxItem(aSourceID));
}

int CFunambolAppView::appendSourcePane(ListboxItem* item) {

    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    int pos = itemList->Count();
    
    refreshPane(item, pos);
    return 0;
}

int CFunambolAppView::removeSourcePane(const TInt aSourceID) {
    
    for (int i=0; i<iListboxItems.size(); i++) {
        ListboxItem* item = (ListboxItem*)iListboxItems[i];
        if (item->getSourceId() == aSourceID) {
            LOG.debug("removing source pane %s", getSourceNameFromID(aSourceID).c_str());
            return removeSourcePane(item, i);
        }
    }
    // not found
    LOG.debug("removeSourcePane: source pane %s not found, exiting", getSourceNameFromID(aSourceID).c_str());
    return 1;
}

int CFunambolAppView::removeSourcePane(ListboxItem* item, const int index) 
{
    MDesCArray* textArray = iListBox->Model()->ItemTextArray();
    CDesCArray* itemList  = static_cast<CDesCArray*>(textArray);
    
    if (GetCurrentItemSelected() >= index) {
        // to avoid errors if the focus is on this item or after
        iListBox->SetCurrentItemIndex(0);
    }

    iCriticalListboxText.Wait();
    
    itemList->Delete(index);
    iListBox->HandleItemAdditionL();
    
    iCriticalListboxText.Signal();
    return 0;
}

ListboxItem* CFunambolAppView::getListboxItem(const StringBuffer& sourceName)
{
    TInt sourceID = getSourceIDFromName(sourceName);
    if (sourceID == KErrNotFound) {
        return NULL;
    }
    
    return getListboxItem(sourceID);
}
*/
