/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "base/SymbianLog.h"
#include "base/FConnection.h"
#include "MediaOptionsSettingList.h"
#include "FunambolAppUi.h"
#include "Funambol.hrh"
#include "utils.h"

using namespace Funambol;


CMediaOptionsSettingList::CMediaOptionsSettingList(const TInt sourceID) : CCommonOptionsSettingList(), 
                                                                         iSourceID(sourceID)
{
    iSyncMode            = 0;
    iMemoryInUseBinary   = EMemoryCardStorage;
    iMemoryInUseMultiple = EMemoryCardStorage;
    iIncludeOlderPictures = 0;
    iIncludeOlderVideos   = 0;
}



CAknSettingItem* CMediaOptionsSettingList::CreateSettingItemL(TInt aIdentifier)
{
    CAknSettingItem* settingItem = NULL;
    switch (aIdentifier)
    {
        case ESettingsListSyncMode:
            settingItem = new (ELeave)CEasyEnumeratedPopupSettingItem(aIdentifier, iSyncMode);
            break;
        case ESettingListMemoryInUse: 
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iMemoryInUseBinary);
            break;
        case ESettingListMassStorageMemoryInUse: 
            settingItem = new (ELeave)CAknEnumeratedTextPopupSettingItem(aIdentifier, iMemoryInUseMultiple);
            break;
        case ESettingListIncludeOlderPictures:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iIncludeOlderPictures);
            break;
        case ESettingListIncludeOlderVideos:
            settingItem = new (ELeave)CEasyBinaryPopupSettingItem(aIdentifier, iIncludeOlderVideos);
            break;
    }
    return settingItem;
}



void CMediaOptionsSettingList::ConstructSyncModeList(ArrayList* syncModeLabels)
{
    //LOG.debug("CMediaOptionsSettingList::ConstructSyncModeList");
    
    // Find the SyncMode item.
    CAknEnumeratedTextPopupSettingItem* syncModePopup;
    syncModePopup = (CAknEnumeratedTextPopupSettingItem*)findItem(ESettingsListSyncMode); 
    if (!syncModePopup) {
        LOG.error("SyncMode popup NULL, exit");
        return;
    }
    
    // Set the array of values
    CArrayPtr<CAknEnumeratedText>* syncModeValues = syncModePopup->EnumeratedTextArray();
    syncModeValues->ResetAndDestroy();
    
    if (!syncModeLabels) {
        return;
    }
    
    for (int i=0; i<syncModeLabels->size(); i++) {
        StringBuffer* label = (StringBuffer*)syncModeLabels->get(i);
        if (label) {
            HBufC* tmp = stringBufferToNewBuf(*label);
            CAknEnumeratedText* text = new CAknEnumeratedText(i, tmp);
            syncModeValues->AppendL(text);
        }
    }

    // Tell the control to load the value in
    syncModePopup->LoadL();
    //LOG.debug("Number of syncmode items = %d", syncModeValues->Count());
}

